﻿// <copyright file="HyperlinkInfo.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Infrastructure
{
    #region Using Statements
    using System;
    using Csla;
    using Csla.Validation;
    #endregion

    /// <summary>
    /// Credentials type.
    /// </summary>
    [Serializable]
    public class HyperlinkInfo : BusinessBase<HyperlinkInfo>
    {                 
        #region Constants and Enums
        #endregion

        #region Instance and Shared Fields
        /// <summary>
        /// Declaration for UrlProperty.
        /// </summary>
        private static PropertyInfo<string> uriProperty = RegisterProperty(typeof(HyperlinkInfo), new PropertyInfo<string>("Uri"));

        /// <summary>
        /// Declaration for TextProperty.
        /// </summary>
        private static PropertyInfo<string> textProperty = RegisterProperty(typeof(HyperlinkInfo), new PropertyInfo<string>("Text"));
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the HyperlinkInfo class.
        /// </summary>
        public HyperlinkInfo()
        {
            this.MarkAsChild();

            this.ValidationRules.CheckRules();
        }

        /// <summary>
        /// Initializes a new instance of the HyperlinkInfo class.
        /// </summary> 
        /// <param name="uri">url parameter</param>
        /// <param name="text">text parameter</param>
        public HyperlinkInfo(string uri, string text)
            : this()
        {
            this.Uri = uri;
            this.Text = text;

            this.MarkOld();
        }
        #endregion

        #region Delegates and Events
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets Uri.
        /// </summary>        
        public string Uri
        {
            get
            {
                this.CanReadProperty(uriProperty.Name, true);
                return this.GetProperty(uriProperty);
            }

            set
            {
                this.CanWriteProperty(uriProperty.Name, true);
                if (!this.GetProperty(uriProperty).Equals(value))
                {
                    this.SetProperty(uriProperty, value);
                    this.PropertyHasChanged(uriProperty.Name);
                }
            }
        }

        /// <summary>
        /// Gets or sets Text.
        /// </summary>        
        public string Text
        {
            get
            {
                this.CanReadProperty(textProperty.Name, true);
                return this.GetProperty(textProperty);
            }

            set
            {
                this.CanWriteProperty(textProperty.Name, true);
                if (!this.GetProperty(textProperty).Equals(value))
                {
                    this.SetProperty(textProperty, value);
                    this.PropertyHasChanged(textProperty.Name);
                }
            }
        }
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// UrlValidator.
        /// Called by BusinessBase RulesEngine.
        /// </summary> 
        /// <typeparam name="T">Type parameter</typeparam>
        /// <param name="target">target parameter</param>
        /// <param name="e">RuleArgs parameter</param>        
        /// <returns>True when validation passes</returns>
        public static bool UrlValidator<T>(T target, RuleArgs e) where T : HyperlinkInfo
        {
            System.Uri uri = null;
            if (System.Uri.TryCreate(target.Uri, UriKind.Absolute, out uri))
            {
                return true;
            }
            else
            {
                e.Description = Properties.Resources.ValidationUriMessage;
                return false;
            }                                                                       
        }

        /// <summary>
        /// Map Fields. 
        /// </summary>  
        /// <param name="hyperlinkInfo">HyperlinkInfo parameter</param>
        public void MapFields(HyperlinkInfo hyperlinkInfo)
        {
            this.Uri = hyperlinkInfo.Uri;
            this.Text = hyperlinkInfo.Text;

            this.ValidationRules.CheckRules();
        }        
        #endregion

        #region Base Class Overrides
        /// <summary>
        /// Add Business Rules. Defines basic validation rules to execute against specified fields.         
        /// </summary>
        protected override void AddBusinessRules()
        {
            // Url.
            this.ValidationRules.AddRule<HyperlinkInfo>(UrlValidator, uriProperty);

            // Text
            ValidationRules.AddRule(CommonRules.StringRequired, textProperty);
        }

        /// <summary>
        /// Override for GetIdValue.
        /// </summary>   
        /// <returns>Id Value for this instance</returns>       
        protected override object GetIdValue()
        {
            return this.GetProperty(uriProperty);
        }
        #endregion

        #region Private and Protected Methods
        #endregion

        #region Event Handlers
        #endregion
    }
}
