﻿// <copyright file="NotesDialogView.xaml.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Notes.Views
{
    #region Using Statements
    using System;
    using System.ComponentModel;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Documents;
    using System.Windows.Input;
    using GoalBook.Infrastructure.Events;
    using GoalBook.Infrastructure.Interfaces;
    #endregion

    /// <summary>
    /// Interaction logic for NotesDialogView.xaml
    /// </summary>
    public partial class NotesDialogView : UserControl, IErrorInfoDialogView
    {        
        #region Constants and Enums
        #endregion

        #region Instance and Shared Fields
        /// <summary>
        /// Declaration for ErrorInfoChangedEventHandler.
        /// </summary>
        private ErrorInfoChangedEventHandler errorInfoChangedEventHandler;        
        #endregion

        #region Constructors        
        /// <summary>
        /// Initializes a new instance of the NotesDialogView class.
        /// </summary>        
        public NotesDialogView()
        {
            this.InitializeComponent();            
        }
        #endregion

        #region Delegates and Events
        /// <summary>
        /// Event indicating that the ErrorInfo property has changed.
        /// </summary>
        public event ErrorInfoChangedEventHandler ErrorInfoChanged
        {
            add { this.errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Combine(this.errorInfoChangedEventHandler, value); }
            remove { this.errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Remove(this.errorInfoChangedEventHandler, value); }
        }

        /// <summary>
        /// Declaration for NoteEditorTextChanged.
        /// </summary>
        public event EventHandler NoteEditorTextChanged;

        /// <summary>
        /// Declaration for NoteEditorSelectionChanged.
        /// </summary>
        public event EventHandler NoteEditorSelectionChanged;

        /// <summary>
        /// Declaration for NoteEditorFocusChanged.
        /// </summary>
        public event EventHandler NoteEditorFocusChanged;

        /// <summary>
        /// Declaration for NoteEditorInsertHyperlink.
        /// </summary>
        public event EventHandler NoteEditorInsertHyperlink;

        /// <summary>
        /// Declaration for NoteEditorPaste.
        /// </summary>
        public event EventHandler NoteEditorPaste;

        /// <summary>
        /// Declaration for NoteEditorPreviewEnter.
        /// </summary>
        public event EventHandler NoteEditorPreviewEnter;
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets a value indicating whether IsDocumentLoaded.
        /// </summary>
        public bool IsDocumentLoaded { get; set; }        
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// Raise ErrorInfo Changed.
        /// </summary>
        /// <param name="info">IDataErrorInfo parameter</param>
        public void RaiseErrorInfoChanged(IDataErrorInfo info)
        {
            this.OnErrorInfoChanged(new ErrorInfoChangedEventArgs(info));
        }
        #endregion

        #region Base Class Overrides
        #endregion

        #region Private and Protected Methods
        /// <summary>
        /// On ErrorInfo Changed.
        /// </summary>
        /// <param name="args">ErrorInfoChangedEventArgs parameter</param>
        private void OnErrorInfoChanged(ErrorInfoChangedEventArgs args)
        {
            if (this.errorInfoChangedEventHandler != null)
            {
                this.errorInfoChangedEventHandler(args);
            }
        }

        /// <summary>
        /// Raise the NoteEditorTextChanged event.
        /// </summary>
        private void OnNoteEditorTextChanged()
        {
            EventHandler noteEditorTextChanged = this.NoteEditorTextChanged;
            if (noteEditorTextChanged != null)
            {
                noteEditorTextChanged(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Raise the NoteEditorSelectionChanged event.
        /// </summary>
        private void OnNoteEditorSelectionChanged()
        {
            EventHandler noteEditorSelectionChanged = this.NoteEditorSelectionChanged;
            if (noteEditorSelectionChanged != null)
            {
                noteEditorSelectionChanged(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Raise the OnNoteEditorFocusChanged event.
        /// </summary>
        private void OnNoteEditorFocusChanged()
        {
            EventHandler noteEditorFocusChanged = this.NoteEditorFocusChanged;
            if (noteEditorFocusChanged != null)
            {
                noteEditorFocusChanged(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Raise the NoteEditorInsertHyperlink event.
        /// </summary>
        private void OnNoteEditorInsertHyperlink()
        {
            EventHandler noteEditorInsertHyperlink = this.NoteEditorInsertHyperlink;
            if (noteEditorInsertHyperlink != null)
            {
                noteEditorInsertHyperlink(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Raise the NoteEditorPaste event.
        /// </summary>
        private void OnNoteEditorPaste()
        {
            EventHandler noteEditorPaste = this.NoteEditorPaste;
            if (noteEditorPaste != null)
            {
                noteEditorPaste(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Raise the NoteEditorPreviewEnter event.
        /// </summary>
        private void OnNoteEditorPreviewEnter()
        {
            EventHandler noteEditorPreviewEnter = this.NoteEditorPreviewEnter;
            if (noteEditorPreviewEnter != null)
            {
                noteEditorPreviewEnter(this, EventArgs.Empty);
            }
        }               
        #endregion
        
        #region Event Handlers
        /// <summary>
        /// Handle UserControl_Loaded event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.Focus();
            this.textBoxTitle.Focus();         
        }
                                
        /// <summary>
        /// Handle NoteEditor_PreviewKeyDown event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">KeyEventArgs parameter</param>
        private void NoteEditor_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyboardDevice.Modifiers == ModifierKeys.Control)
            {
                switch (e.Key)
                {
                    case Key.E: // Align Center
                    case Key.J: // Align Justify
                    case Key.L: // Align Left
                    case Key.R: // Align Right
                    case Key.OemOpenBrackets: // Decrease FontSize
                    case Key.OemCloseBrackets: // Increase FontSize
                    case Key.OemPlus: // ToggleSubScript                    
                    case Key.U: // Toggle Underline
                        e.Handled = true;                                           
                        break;
                    case Key.V: // Paste                        
                        this.OnNoteEditorPaste();
                        e.Handled = true;                                                                        
                        break;
                    default:
                        break;
                }
            }
            else if (e.KeyboardDevice.Modifiers == (ModifierKeys.Control | ModifierKeys.Shift))
            {
                switch (e.Key)
                {                    
                    case Key.OemPlus: // ToggleSuperScript                    
                        e.Handled = true;
                        break;
                    default:
                        break;
                }
            }
            else if (e.Key == Key.Enter)
            {                
                this.OnNoteEditorPreviewEnter();                                 
            }
        }
                
        /// <summary>
        /// Handle NoteEditor_KeyDown event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">KeyEventArgs parameter</param>
        private void NoteEditor_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyboardDevice.Modifiers == ModifierKeys.Control)
            {
                switch (e.Key)
                {
                    case Key.K: // NoteEditorInsertHyperlink                        
                        this.OnNoteEditorInsertHyperlink();                                                
                        e.Handled = true;
                        break;                    
                }
            }            
        }
                
        /// <summary>
        /// Handle NoteEditor_TextChanged event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">TextChangedEventArgs parameter</param>
        private void NoteEditor_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (this.IsDocumentLoaded)
            {
                // Raise to presenter
                this.OnNoteEditorTextChanged();
            }
        }
        
        /// <summary>
        /// Handle NoteEditor_FocusChanged event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">TextChangedEventArgs parameter</param>
        private void NoteEditor_FocusChanged(object sender, RoutedEventArgs e)
        {
            // Raise to presenter
            this.OnNoteEditorFocusChanged();
        }

        /// <summary>
        /// Handle NoteEditor_SelectionChanged event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void NoteEditor_SelectionChanged(object sender, RoutedEventArgs e)
        {
            // Raise to presenter
            this.OnNoteEditorSelectionChanged();
        }

        /// <summary>
        /// Handle NoteEditor_PreviewMouseRightButtonUp event.
        /// </summary>
        /// <param name="sender">sebder parameter</param>
        /// <param name="e">MouseButtonEventArgs parameter</param>
        private void NoteEditor_PreviewMouseRightButtonUp(object sender, MouseButtonEventArgs e)
        {
            // Cancels context menu.
            e.Handled = true;
        }        
        #endregion
    }
}
