// <copyright file="TasksDialogView.xaml.cs" company="GoalBook"> 
//    Copyright  2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Tasks.Views
{
    #region Using Statements
    using System;
    using System.ComponentModel;
    using System.Windows;
    using System.Windows.Controls;
    using GoalBook.Infrastructure.Events;
    using GoalBook.Infrastructure.Interfaces;
    using Microsoft.Windows.Controls;    
    #endregion

    /// <summary>
    /// Interaction logic for TasksDialogView.xaml
    /// </summary>    
    public partial class TasksDialogView : UserControl, IErrorInfoDialogView
    {
        #region Declarations 
        /// <summary>
        /// Declaration for ErrorInfoChangedEventHandler.
        /// </summary>        
        private static ErrorInfoChangedEventHandler errorInfoChangedEventHandler;
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the TasksDialogView class.
        /// </summary>
        public TasksDialogView()
        {            
            InitializeComponent();
        }
        #endregion

        #region Events 
        /// <summary>
        /// Event indicating that the ErrorInfo property has changed.
        /// </summary>
        public event ErrorInfoChangedEventHandler ErrorInfoChanged
        {
            add { errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Combine(errorInfoChangedEventHandler, value); }
            remove { errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Remove(errorInfoChangedEventHandler, value); }
        }       
        #endregion

        #region Public Members
        /// <summary>
        /// Raise ErrorInfo Changed event. 
        /// </summary> 
        /// <param name="info">IDataErrorInfo parameter</param>
        public void RaiseErrorInfoChanged(IDataErrorInfo info)
        {
            this.OnErrorInfoChanged(new ErrorInfoChangedEventArgs(info));
        }
        #endregion

        #region Private Members
        /// <summary>
        /// Raise the OnErrorInfoChanged event.
        /// </summary>
        /// <param name="args">ErrorInfoChangedEventArgs parameter</param>
        private void OnErrorInfoChanged(ErrorInfoChangedEventArgs args)
        {
            if (errorInfoChangedEventHandler != null)
            {
                errorInfoChangedEventHandler(args);
            }
        }
        #endregion

        #region EventHandlers
        /// <summary>
        /// Handle UserControl_Loaded event.
        /// </summary>  
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.Focus();
            this.textBoxTitle.Focus();

            DateTime? dueDate = (this.DataContext as TasksDialogViewPresentationModel).EditTask.Due;

            // Initialise the DatePicker control.
            if (dueDate.HasValue)
            {
                this.dueDatePicker.IsTodayHighlighted = false;
                this.dueDatePicker.DisplayDateStart = dueDate.Value > DateTime.Today ? DateTime.Today : dueDate.Value;
                
                if (dueDate.Value < DateTime.Today.AddDays(-1))
                {   
                    // When due date is in the past then blackout the daterange until today.          
                    this.dueDatePicker.BlackoutDates.Add(new CalendarDateRange(dueDate.Value.AddDays(1), DateTime.Today.AddDays(-1)));                   
                }
            }
            else
            {
                this.dueDatePicker.DisplayDateStart = DateTime.Today;
                this.dueDatePicker.IsTodayHighlighted = true;
            }
        }               
        #endregion
    }
}
